/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _ASSOCSRCINFO_H_
#define _ASSOCSRCINFO_H_

#include "Object.h"
#include "ObjectParser.h"
#include "misc.h"
#include <cmath>

using namespace FD;

class AssocSrcInfo : public Object{ 

 public:
  
  typedef struct tag_SrcInfo_Audio {
    int id;          // Object ID
    float obj[3];    // 3D position of the object from the base sensor (x, y, z)
    float sensor[6]; // 3D position and orientation of the base sensor (x, y, z, roll, pitch, yaw)
    float power;     // power of sound source
  } SrcInfo_Audio;

  typedef struct tag_SrcInfo_Camera {
    int id;          // Object ID
    float obj[3];    // 3D position of the object from the base sensor (x, y, z)
    float sensor[6]; // 3D position and orientation of the base sensor (x, y, z, roll, pitch, yaw)
    int img_size[2]; // Pixel size of the sensor image
    int roi_c[2];    // Center of the ROI in the image of base camera
    int roi_xy[2];   // Size of the ROI in the image of base camera
  } SrcInfo_Camera;

  typedef struct tag_SrcInfo_TOF {
    int id;          // Object ID
    float obj[3];    // 3D position of the object from the base sensor (x, y, z)
    float sensor[6]; // 3D position and orientation of the base sensor (x, y, z, roll, pitch, yaw)
    int img_size[2]; // Pixel size of the sensor image
    int roi_c[2];    // Center of the ROI in the image of base camera
    int roi_xy[2];   // Size of the ROI in the image of base camera
  } SrcInfo_TOF;

  typedef struct tag_SrcInfo_Thermo {
    int id;          // Object ID
    float obj[3];    // 3D position of the object from the base sensor (x, y, z)
    float sensor[6]; // 3D position and orientation of the base sensor (x, y, z, roll, pitch, yaw)
    int img_size[2]; // Pixel size of the sensor image
    int roi_c[2];    // Center of the ROI in the image of base camera
    int roi_xy[2];   // Size of the ROI in the image of base camera
  } SrcInfo_Thermo;

  bool valid_Audio; 
  bool valid_Camera;
  bool valid_TOF;
  bool valid_Thermo;
  SrcInfo_Audio src_Audio;
  SrcInfo_Camera src_Camera;
  SrcInfo_TOF src_TOF;
  SrcInfo_Thermo src_Thermo;
 
  AssocSrcInfo(){
  }

  AssocSrcInfo(const AssocSrcInfo& input) {
    valid_Audio  = input.valid_Audio;
    valid_Camera = input.valid_Camera;
    valid_TOF    = input.valid_TOF;
    valid_Thermo = input.valid_Thermo;
  }
      
  ~AssocSrcInfo(){
  }

  virtual ObjectRef clone()
  {
    return ObjectRef(new AssocSrcInfo(*this));
  }
		
  /**Serialize (binary) the object to a stream*/
  virtual void serialize(std::ostream &out) const {
  }
  
  /**Unserialize (binary) the object from a stream*/
  virtual void unserialize(std::istream &in){
  }
		
  /**Generic print function*/
  virtual void printOn(std::ostream &out=std::cout) const {
    out << "<AssocSrcInfo " << std::endl;
    out << "<Audio valid " << valid_Audio << " >" << std::endl;
    out << "<Camera valid " << valid_Camera << " >" << std::endl;
    out << "<TOF valid " << valid_TOF << " >" << std::endl;
    out << "<Thermo valid " << valid_Thermo << " >" << std::endl;
    out << ">" << std::endl;
  }
     
  /**Generic read function*/
  virtual void readFrom(std::istream &in=std::cin){
  }      
      
};

#endif
